@@ -17,6 +17,8 @@ def create(
     cutoff_dt: datetime.datetime | None = None,
     embedding_cols: list[list[str]] | None = None,
     allowed_models: set[str] | None = None,
+    max_false_positive_reduction_steps: int | None = None,
+    correlation_chunk_size: int | None = None,
 ) -> Trainer:
     """Create a trainer."""
     return Trainer(
@@ -29,4 +31,6 @@ def create(
         cutoff_dt=cutoff_dt,
         embedding_cols=embedding_cols,
         allowed_models=allowed_models,
+        max_false_positive_reduction_steps=max_false_positive_reduction_steps,
+        correlation_chunk_size=correlation_chunk_size,
     )
