@@ -42,6 +42,7 @@ _VALIDATION_SIZE_KEY = "validation_size"
 _IDX_USR_ATTR_KEY = "idx"
 _DT_COLUMN_KEY = "dt_column"
 _MAX_FALSE_POSITIVE_REDUCTION_STEPS_KEY = "max_false_positive_reduction_steps"
+_CORRELATION_CHUNK_SIZE_KEY = "correlation_chunk_size"
 _BAD_OUTPUT = -1000.0
 
 
@@ -53,8 +54,9 @@ def _assign_bin(timestamp, bins: list[datetime.datetime]) -> int:
 
 
 def _best_trial(study: optuna.Study) -> optuna.trial.FrozenTrial:
-    best_brier = min(study.best_trials, key=lambda t: t.values[1])
-    return best_brier
+    return study.best_trial
+    # best_brier = min(study.best_trials, key=lambda t: t.values[1])
+    # return best_brier
 
 
 class Trainer(Fit):
@@ -75,6 +77,7 @@ class Trainer(Fit):
         embedding_cols: list[list[str]] | None = None,
         allowed_models: set[str] | None = None,
         max_false_positive_reduction_steps: int | None = None,
+        correlation_chunk_size: int | None = None,
     ):
         tqdm.tqdm.pandas()
 
@@ -129,6 +132,8 @@ class Trainer(Fit):
                     max_false_positive_reduction_steps = params.get(
                         _MAX_FALSE_POSITIVE_REDUCTION_STEPS_KEY
                     )
+                if correlation_chunk_size is None:
+                    correlation_chunk_size = params.get(_CORRELATION_CHUNK_SIZE_KEY)
         else:
             with open(params_file, "w", encoding="utf8") as handle:
                 validation_size_value = None
@@ -160,6 +165,7 @@ class Trainer(Fit):
                         _VALIDATION_SIZE_KEY: validation_size_value,
                         _DT_COLUMN_KEY: dt_column,
                         _MAX_FALSE_POSITIVE_REDUCTION_STEPS_KEY: max_false_positive_reduction_steps,
+                        _CORRELATION_CHUNK_SIZE_KEY: correlation_chunk_size,
                     },
                     handle,
                 )
@@ -173,6 +179,7 @@ class Trainer(Fit):
         self.embedding_cols = embedding_cols
         self._allowed_models = allowed_models
         self._max_false_positive_reduction_steps = max_false_positive_reduction_steps
+        self._correlation_chunk_size = correlation_chunk_size
 
     def _provide_study(self, column: str) -> optuna.Study:
         storage_name = f"sqlite:///{self._folder}/{column}/{_STUDYDB_FILENAME}"
@@ -187,7 +194,7 @@ class Trainer(Fit):
             load_if_exists=True,
             sampler=restored_sampler,
             directions=[
-                optuna.study.StudyDirection.MAXIMIZE,
+                # optuna.study.StudyDirection.MAXIMIZE,
                 optuna.study.StudyDirection.MINIMIZE,
             ],
         )
@@ -229,7 +236,8 @@ class Trainer(Fit):
                 save: bool,
                 split_idx: datetime.datetime,
                 no_evaluation: bool,
-            ) -> tuple[float, float]:
+                # ) -> tuple[float, float]:
+            ) -> float:
                 print(f"Beginning trial for: {split_idx.isoformat()}")
                 trial.set_user_attr(_IDX_USR_ATTR_KEY, split_idx.isoformat())
                 folder = os.path.join(
@@ -246,7 +254,9 @@ class Trainer(Fit):
                                 "Found trial %d previously executed, skipping...",
                                 trial.number,
                             )
-                            return trial_info["output"]
+                            # return tuple(trial_info["output"])
+                            return tuple(trial_info["output"])[0]
+                        print("Retraining for different trial number.")
 
                 train_dt_index = dt_index[: len(x)]
                 x_train = x[train_dt_index < split_idx]  # type: ignore
@@ -265,12 +275,15 @@ class Trainer(Fit):
                         if new_folder:
                             os.removedirs(folder)
                         logging.warning("Y train only contains 1 unique datapoint.")
-                        return _BAD_OUTPUT, -_BAD_OUTPUT
+                        # return _BAD_OUTPUT, -_BAD_OUTPUT
+                        return -_BAD_OUTPUT
                     print(f"Windowing took {time.time() - start_windower}")
 
                     # Perform common reductions
                     start_reducer = time.time()
-                    reducer = CombinedReducer(self.embedding_cols)
+                    reducer = CombinedReducer(
+                        self.embedding_cols, self._correlation_chunk_size
+                    )
                     reducer.set_options(trial, x)
                     x_train = reducer.fit_transform(x_train, y=y_train)
                     x_test = reducer.transform(x_test)
@@ -367,18 +380,20 @@ class Trainer(Fit):
                             json.dump(
                                 {
                                     "number": trial.number,
-                                    "output": output,
+                                    "output": [output, loss],
                                 },
                                 handle,
                             )
 
-                    return output, loss
+                    # return output, loss
+                    return loss
                 except WavetrainException as exc:
                     print(str(exc))
                     logging.warning(str(exc))
                     if new_folder:
                         os.removedirs(folder)
-                    return _BAD_OUTPUT, -_BAD_OUTPUT
+                    # return _BAD_OUTPUT, -_BAD_OUTPUT
+                    return -_BAD_OUTPUT
 
             start_validation_index = (
                 dt_index.to_list()[-int(len(dt_index) * self._validation_size) - 1]
@@ -399,7 +414,8 @@ class Trainer(Fit):
                 ].to_list()[0]
             )
 
-            def test_objective(trial: optuna.Trial) -> tuple[float, float]:
+            # def test_objective(trial: optuna.Trial) -> tuple[float, float]:
+            def test_objective(trial: optuna.Trial) -> float:
                 return _fit(
                     trial,
                     test_df,
@@ -481,8 +497,11 @@ class Trainer(Fit):
                 if test_idx < start_validation_index:
 
                     def validate_objctive(
-                        trial: optuna.Trial, idx: datetime.datetime, series: pd.Series
-                    ) -> tuple[float, float]:
+                        trial: optuna.Trial,
+                        idx: datetime.datetime,
+                        series: pd.Series,
+                        # ) -> tuple[float, float]:
+                    ) -> float:
                         return _fit(trial, test_df.copy(), series, False, idx, False)
 
                     study.optimize(
@@ -508,15 +527,15 @@ class Trainer(Fit):
                 last_processed_dt = test_idx
 
             target_names = ["F1", "Brier"]
-            fig = optuna.visualization.plot_pareto_front(
-                study, target_names=target_names
-            )
-            fig.write_image(
-                os.path.join(column_dir, "pareto_frontier.png"),
-                format="png",
-                width=800,
-                height=600,
-            )
+            # fig = optuna.visualization.plot_pareto_front(
+            #    study, target_names=target_names
+            # )
+            # fig.write_image(
+            #    os.path.join(column_dir, "pareto_frontier.png"),
+            #    format="png",
+            #    width=800,
+            #    height=600,
+            # )
             for target_name in target_names:
                 fig = optuna.visualization.plot_param_importances(
                     study, target=lambda t: t.values[0], target_name=target_name
@@ -536,7 +555,7 @@ class Trainer(Fit):
 
         return self
 
-    def transform(self, df: pd.DataFrame) -> pd.DataFrame:
+    def transform(self, df: pd.DataFrame, optimistic: bool = False) -> pd.DataFrame:
         """Predict the expected values of the data."""
         tqdm.tqdm.pandas(desc="Inferring...")
         input_df = df.copy()
@@ -577,13 +596,27 @@ class Trainer(Fit):
                     if self._dt_column is None
                     else pd.DatetimeIndex(pd.to_datetime(group[self._dt_column]))
                 )
-                filtered_dates = [x for x in dates if x < group_dt_index.max()]
+
+                max_date = group_dt_index.max()
+                if not optimistic:
+                    if isinstance(self._validation_size, float):
+                        max_date = group_dt_index[
+                            -int(len(group_dt_index) * self._validation_size)
+                        ]
+                    elif isinstance(self._validation_size, int):
+                        max_date = group_dt_index[-self._validation_size]
+                    else:
+                        max_date = max_date - self._validation_size
+
+                filtered_dates = [x for x in dates if x < max_date]
                 if not filtered_dates:
                     filtered_dates = [dates[-1]]
                 date_str = dates[-1].isoformat()
                 folder = os.path.join(column_path, date_str)
 
-                reducer = CombinedReducer(self.embedding_cols)
+                reducer = CombinedReducer(
+                    self.embedding_cols, self._correlation_chunk_size
+                )
                 reducer.load(folder)
 
                 model = ModelRouter(None, None)
@@ -626,7 +659,9 @@ class Trainer(Fit):
 
         return df
 
-    def feature_importances(self) -> dict[str, dict[str, float]]:
+    def feature_importances(
+        self, df: pd.DataFrame
+    ) -> dict[str, tuple[dict[str, float], list[dict[str, float]]]]:
         """Find the feature importances for the rolling models."""
         feature_importances = {}
 
@@ -641,7 +676,7 @@ class Trainer(Fit):
                 try:
                     model = ModelRouter(None, None)
                     model.load(date_path)
-                    feature_importances[date_str] = model.feature_importances
+                    feature_importances[date_str] = model.feature_importances(df)
                 except FileNotFoundError as exc:
                     logging.warning(str(exc))
 
